


############## Stat descriptives multidimensionnelle ##########################################################
## - utilises quand le nombre de variables p mesures est > 2                                     ############
## - BUT: rsumer l'information apporte  une reprsentation graphique  2 dimensions             ############
## - diffrentes mthodes suivant le type de donnes  analyser                                    ############
###############################################################################################################


 
############################# Analyse en composantes principale (ACP) ######################################
## p > 2 variables quantitatives mesures sur n individus (tableau de dimension n x p) #####################
## l'analyse vise  reprsenter graphiquement: - la ressemblance entre individus       #####################
## 							     - les corrlations entre variables      #####################
## transformer des variables "corrles": les "composantes principales", ou axes, rduisent ################
## l'information en un nombre plus petit de variables (les composantes)				  ################
## qui sont indpendantes les unes des autres					                    ################
## (utilise la covariance entre variables)                                                  ################
############################################################################################################




################################# ACP EXEMPLE 1 ############################################################
## faire l'ACP du dataset "gene_expression.txt"                                          ###################
## effet de diffrentes conditions exprimentales (concentration en eliciteur)           ###################
## sur l'expression de diffrents gnes chez Arabidopsis                                 ###################


genes = read.table("gene_expression.txt",header=T)
genes
# Quels sont les individus ? les variables ?
# Que signifie header=T dans la commande read.table ?
attach(genes)
names(genes)
class(genes)
# Qu'est-ce qu'une data.frame ?
summary(genes)
dim(genes)
boxplot(genes,las=3,ylab="gene expression")
# Qu'observe-t-on par rapport au plages de valeurs des diffrentes variables ?
var(X4CL)
var(CCoAOMT)

pairs(genes,pch=16)
cor(genes)
# Quelles sont les variables corrles ? anti-corrles ?
acp=prcomp(genes,scale=T)

acp
# cart-type des axes (= sqrt(eigen(cor(metadata))$values), racine carre des valeurs propres de la matrice de corrlation)
# valeur propre = variance exprime par l'axe
acp$sdev

# une matrice dont les colonnes sont les vecteurs propres (= coordonnes) des variables (vecteur propre = direction de l'axe)
acp$rotation

# une matrice des coordonnes des individus sur chaque axe ("matrice des valeurs centres rduites" X "matrice des vecteurs propres")
acp$x

# contribution des axes
summary(acp)

# La mme chose manuellement
round(  acp$sdev**2/sum(acp$sdev**2)  ,4)
round(  cumsum( acp$sdev**2)/sum(acp$sdev**2) ,4)
# rappel sur l'affichage d'un camembert
pie(acp$sdev**2)

biplot(acp)
# biplot des individus et des variables (sur un axe, les individus/variables de coordonnes proches sont similaires)         
par(mfrow=c(1,2))
plot(acp$x[,1:2], pch="",main="reprsentation des distances entre individus")
text(acp$x[,1:2], labels=abbreviate(row.names(genes)), col="blue")
plot(acp$rotation[,1:2], pch="", xlim=c(-1,1), ylim=c(-1,1),main="reprsentation des corrlations entre variables")
text(acp$rotation[,1:2], labels=colnames(genes), col="red")

# importance de la normalisation
par(mfrow=c(1,2))
acp1=prcomp(genes,scale=F)
biplot(acp1,scale=0,main="ACP non norme")
acp2=prcomp(genes,scale=T)
biplot(acp2,scale=0,main="ACP norme")
############################################################################################################




############################# Analyse Factorielle des correspondances (AFC) ################################
## Tableau de contingence de 2 variables qualitatives  p et q modalits chacune (p x q) ###################
## visualisation en deux dimensions des tableaux de contingences par analyse factorielle ###################
## l'AFC peut tre vue comme une ACP avec une distance particulire : la distance du chi^2 #################
## le tableau original est dcompose en une somme de tableaux/matrices                    #################
## qui sont chacun le produit de facteurs simples: on reprsente des facteurs ##############################
############################################################################################################


#################################  AFC EXEMPLE  ############################################################
## recherche de similitudes (correspondances) entre diffrentes espces de plantes en terme d'abondance  ###
## de diffrents microorganismes associs                                                  #################


species=read.table("species.txt",header=T)
species
attach(species)
names(species)
class(species)
summary(species)
dim(species)
cor(species)
#test de chi2 d'indpendance (entre espces de plante et type de microorganisme)
chisq.test(as.matrix(species))

############## reprsentation des donnes avec la fonction table.value()du package ade4 ####################
library(ade4)
table.value(species, grid=T)

############## analyse factorielle des correspondances avec la fonction CA() du package FactoMineR #########
library(FactoMineR)
# donne une liste de diffrents rsultats de l'analyse + un graphique
afc <- CA(species, ncp=3)
#diffrents rsultats de l'analyse
# valeurs propres, % de variance et % de variance cumulatif
afc$eig

# liste de matrices pour la variable ligne
afc$row
# liste de matrices pour la variable colonne
afc$col

# reprsentation graphique
# plot facteur 1 vs facteur 2	
plot.CA(afc, axes=c(1,2))
# plot facteur 2 vs facteur 3
plot.CA(afc, axes=c(2,3))

############## en 3D si la lib est installe
library(rgl)
plot3d(afc$col$coord[,1],   afc$col$coord[,2], afc$col$coord[,3], col="red")
text3d(afc$col$coord[,1],   afc$col$coord[,2], afc$col$coord[,3], rownames(afc$col$coord), col="red")
points3d(afc$row$coord[,1], afc$row$coord[,2],     afc$row$coord[,3], col="blue")
text3d(afc$row$coord[,1],   afc$row$coord[,2], afc$row$coord[,3], rownames(afc$row$coord), col="blue")

		################################
		#   Liste des microorganismes
		# Sinorhizobium meliloti
		# Aphanomyces euteiches
		# Colletotrichum trifoli
		# Phytophthora sp
		# Magnaporthe
		# Phoma mcdonaldii
		# Ralstonia sp
		# Plasmopara
		# Xanthomonas
		# Frankia
		##############################


############## si package FactomineR non install 

# avec ade4
library(ade4)
afc=dudi.coa(species,scannf=FALSE, nf=3)
par(mfrow=c(1,2))
# axe 1 et 2
plot(afc$co[,1], afc$co[,2], pch="")
text(afc$co[,1], afc$co[,2], rownames(afc$co), col="red")
text(afc$li[,1], afc$li[,2], rownames(afc$li), col="blue")
abline(h=0,v=0, lty=2)
# axe 2 et 3
plot(afc$co[,2], afc$co[,3], pch="")
text(afc$co[,2], afc$co[,3], rownames(afc$co), col="red")
text(afc$li[,2], afc$li[,3], rownames(afc$li), col="blue")
abline(h=0,v=0, lty=2)
# en 3D si rgl
library(rgl)
plot3d(afc$co[,1], afc$co[,2], afc$co[,3], col="red")
text3d(afc$co[,1], afc$co[,2], afc$co[,3], rownames(afc$co), col="red")
points3d(afc$li[,1], afc$li[,2], afc$li[,3], col="blue")
text3d(afc$li[,1], afc$li[,2], afc$li[,3], rownames(afc$li), col="blue")

# pourcentage de variance expliqu par chaque axe
round(afc$eig / sum(afc$eig)*100,2)
# pourcentages cumuls
round( cumsum(afc$eig / sum(afc$eig)*100), 2)
############################################################################################################





############################# Multidimensional scaling (MDS) ###############################################
## c'est une ACP sur tableau de distances, calcul  partir d'un tableau n x p  ############################
## on mesure une distance entre individus base sur les valeurs  chaque variable ##########################
## puis ordination sur matrice de distances                                     ############################
############################################################################################################


################################# MDS EXEMPLE   ############################################################
## diffrences multilocus entre lignes de Medicago truncatula                    ##########################
## (distance inter-individuelle base sur une distance euclidienne, par exemple)  ##########################

geno=read.table("genotypes.txt",header=T)
attach(geno); names(geno);class(geno)
summary(geno);dim(geno)					

d <- dist(geno) # euclidean distances between the rows
fit <- cmdscale(d,eig=TRUE, k=2) # k est le nombre de dimensions
fit # view results

# plot solution 
x <- fit$points[,1]
y <- fit$points[,2]
plot(x, y, xlab="Coordinate 1", ylab="Coordinate 2",main="MDS sur gnotypes multilocus", type="n")
text(x, y, labels = row.names(geno), cex=0.7)

# combien de groupes gntiques identifiez vous?

# en 3D si rgl
fit <- cmdscale(d,eig=TRUE, k=3)
x <- fit$points[,1]
y <- fit$points[,2]
z <- fit$points[,3]
plot3d(x, y, z, xlab="Coordinate 1", ylab="Coordinate 2",main="MDS sur genotypes multilocus", type="n")
text3d(x, y, z, row.names(geno), cex=0.7)
# quelle ligne se distingue maintenant de ses voisins en 3D ?
############################################################################################################















###################################################################################################################################
################################## PROBLEMES SUPPLEMENTAIRES ######################################################################
###################################################################################################################################



################################# ACP EXEMPLE 2 ############################################################
## bactries: variables gnomiques et environnementales ####################################################
## quel est la ressemblance entre les diffrentes espces sur l'ensemble des variables ?      ##############
## quel est le degr de corrlation entre les diffrentes variables ?                         ##############


metadata = read.table("gold.metadata.txt", sep="\t",header=TRUE,row.names=1)
metadata; class(metadata)
attach(metadata); names(metadata)     # on peut accder aux objets en donnant leur nom
metadata=metadata[,-1]                # enlever les variables non numriques

summary(metadata)                     # rsume la dispersion au sein de chaque variable 	
dim(metadata)				  # dimensions (n,p) du tableau de donnes
boxplot(metadata,las=3)
boxplot(metadata[,-c(1:3)],las=3)           # visualisation de l'htrognit des valeurs (Gc_content, Size_kb, latitude ....)

var(Size_kb);var(Orfs)
var(Temperature);var(Chromosome_count)# variances diffrentes, en chiffres

cor(metadata)                         # matrice des corrlations entre les diffrentes variables

acp=prcomp(metadata,scale=T)	  # prcomp = fonction pour raliser l'ACP
                                      # ici la variance des variables diffre fortement -> normalisation approprie
# ou -> prcomp(~ Size_kb + Orfs + Chromosome_count + Gc_content + Latitude + Longitude + Temperature + Sequencing_depth + Disease + intergenic_ratio, data = metadata, scale = T) 

acp
acp$sdev		# cart-type des axes (= sqrt(eigen(cor(metadata))$values), racine carre des valeurs propres de la matrice de corrlation)
				# valeur propre = variance exprime par l'axe				
acp$rotation        # une matrice dont les colonnes sont les vecteurs propres (= coordonnes) des variables (vecteur propre = direction de l'axe)
acp$x               # une matrice des coordonnes des individus sur chaque axe ("matrice des valeurs centres rduites" X "matrice des vecteurs propres")
summary(acp)		# contribution des axes


biplot(acp)	 
# biplot des individus et des variables (sur un axe, les individus/variables de coordonnes proches sont similaires)         
par(mfrow=c(1,2))
plot(acp$x[,1:2], pch="",main="reprsentation des distances entre individus")
text(acp$x[,1:2], labels=abbreviate(row.names(metadata)), col="blue")
plot(acp$rotation[,1:2], pch="", xlim=c(-1,1), ylim=c(-1,1),main="reprsentation des corrlations entre variables")
text(acp$rotation[,1:2], labels=colnames(metadata), col="red")
############################################################################################################
















